<?php

namespace App\Mobile\Synchronization\Controllers;

use App\Domain\Logs\Model\ErrorLog;
use App\Domain\Logs\Service\SaveError;
use App\Http\Controller;
use App\Mobile\Synchronization\Managers\SyncManager;
use App\Mobile\Synchronization\Requests\UploadSyncRequest;
use App\Mobile\Synchronization\Resources\SyncResource;
use Schaffen\Core\Company\Repositories\BranchRepository;
use Schaffen\Core\EnvironmentConfiguration\Repositories\ParameterRepository;
use Schaffen\Core\Form\Repositories\FormRepository;
use Schaffen\Core\Movement\Managers\InventoryControlSchedulingManager;
use Schaffen\Core\Person\Models\Person;
use Schaffen\Core\Person\Repositories\AddressTypeRepository;
use Schaffen\Core\Person\Repositories\PersonRepository;
use Schaffen\Core\Product\Repositories\ProductVarietyPackRepository;
use Schaffen\Core\Visitation\Repository\VisitationReasonRepository;
use Schaffen\ServiceOrder\ServiceOrder\Models\ServiceOrder;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceOrderRepository;
use Schaffen\WarehouseInventory\WarehouseInventory\Repositories\SnapshotInventoryRepository;

class SyncController extends Controller
{
    /**
     * @var PersonRepository
     */
    private PersonRepository $person_repository;

    /**
     * @var ServiceOrderRepository
     */
    private ServiceOrderRepository $service_order_repository;

    /**
     * @var FormRepository
     */
    private FormRepository $form_repository;

    /**
     * @var SnapshotInventoryRepository
     */
    private SnapshotInventoryRepository $snapshot_inventory_repository;

    /**
     * @var BranchRepository
     */
    private BranchRepository $branch_repository;

    /**
     * @var ProductVarietyPackRepository
     */
    private ProductVarietyPackRepository $product_variety_pack_repository;

    /**
     * @var VisitationReasonRepository
     */
    private VisitationReasonRepository $visitation_reason_repository;

    /**
     * @var AddressTypeRepository
     */
    private AddressTypeRepository $address_type_repository;
    /**
     * @var ParameterRepository
     */
    private ParameterRepository $parameter_repository;
    private InventoryControlSchedulingManager $inventory_control_scheduling_manager;

    /**
     * SyncController constructor.
     * @param PersonRepository $person_repository
     * @param ServiceOrderRepository $service_order_repository
     * @param FormRepository $form_repository
     * @param SnapshotInventoryRepository $snapshot_inventory_repository
     * @param BranchRepository $branch_repository
     * @param ProductVarietyPackRepository $product_variety_pack_repository
     * @param VisitationReasonRepository $visitation_reason_repository
     * @param AddressTypeRepository $address_type_repository
     * @param ParameterRepository $parameter_repository
     */
    public function __construct(
        PersonRepository $person_repository,
        ServiceOrderRepository $service_order_repository,
        FormRepository $form_repository,
        SnapshotInventoryRepository $snapshot_inventory_repository,
        BranchRepository $branch_repository,
        ProductVarietyPackRepository $product_variety_pack_repository,
        VisitationReasonRepository $visitation_reason_repository,
        AddressTypeRepository $address_type_repository,
        ParameterRepository $parameter_repository,
        InventoryControlSchedulingManager $inventory_control_scheduling_manager
    )
    {
        $this->person_repository = $person_repository;
        $this->service_order_repository = $service_order_repository;
        $this->form_repository = $form_repository;
        $this->snapshot_inventory_repository = $snapshot_inventory_repository;
        $this->branch_repository = $branch_repository;
        $this->product_variety_pack_repository = $product_variety_pack_repository;
        $this->visitation_reason_repository = $visitation_reason_repository;
        $this->address_type_repository = $address_type_repository;
        $this->parameter_repository = $parameter_repository;
        $this->inventory_control_scheduling_manager = $inventory_control_scheduling_manager;

    }

    /**
     * @return SyncResource
     */
    public function sync(): SyncResource
    {
        try {
            //PESQUISA
            /** @var Person $person */
            $person = auth()->user()->person()->select(['pes_codigo', 'pes_nome'])->first();

            //INPUTS
            $os_filters = [
                "os_recurso" => $person->pes_codigo,
                "not_final_or_executing" => true,
                "import_movel" => true,
                "has_scheduling" => true,
                "has_form" => true,
                "not_executed" => true,
                "schedule_limit" => now()->addWeek(),
            ];

            //PESQUISA
            $service_order = $this->service_order_repository
                ->setOrder('os_data_agendamento', 'asc')
                ->list($os_filters, [
                    'address' => function ($query) {
                        $query->select([
                            'pee_codigo',
                            'pee_logradouro',
                            'pee_numero',
                            'pee_uf',
                            'pee_cidade',
                            'pee_bairro',
                            'pee_complemento',
                            'pee_distancia_km',
                            'pee_latitude',
                            'pee_longitude']);
                    },
                    'client' => function ($query) {
                        $query->select(['pes_codigo', 'pes_nome']);
                    },
                    'form_association' => function ($query) {
                        $query->select(['foa_codigo', 'foa_respostas', 'foa_formeavel_tipo', 'foa_formeavel_codigo', 'foa_fk_form_codigo']);
                    },
                    'forms' => function ($query) {
                        $query->select(['form_codigo as formulario']);
                    },
                    'items' => function ($query) {
                        $query->select(['osps_codigo', 'osps_servico', 'osps_fk_os_codigo', 'osps_quantidade',
                            'osps_fk_pve_codigo', 'osps_valor_unitario']);
                    },
                    'visitation' => function ($query) {
                        $query->select(['vis_codigo', 'vis_fk_mov_codigo', 'vis_visitavel_tipo', 'vis_visitavel_codigo']);
                    },
                    'items.product_variety_pack.measurement_unit' => function ($query) {
                        $query->select(['unm_codigo', 'unm_descricao']);
                    }
                ], false);
            //PESQUISA
            $visitation_reasons = $this->visitation_reason_repository->setSelect(['mov_codigo', 'mov_descricao'])
                ->list([
                    "has_form" => true,
                    "mov_status" => 1,
                ], [
                    'forms' => function ($query) {
                        $query->select(['form_codigo as formulario']);
                    }
                ], false);


            $forms_in = collect([]);
            foreach ($service_order as $so) {
                if ($so->os_data_agendamento < now()->endOfDay()) {
                    $so->os_fk_sto_codigo = ServiceOrder::STATUS_EM_EXECUCAO;
                    //$so->os_planejada = true;
                    $so->save();
                }
                $forms_in = $forms_in->merge($so->forms->pluck('formulario'));
            }
            foreach ($visitation_reasons as $visitation_reason) {
                $forms_in = $forms_in->merge($visitation_reason->forms->pluck('formulario'));
            }

            //INPUTS
            $form_filters = [
                "form_status" => 1,
                "forms_in" => $forms_in,
                "has_fields" => true
            ];

            //PESQUISA
            $forms = $this->form_repository->setSelect(['form_codigo', 'form_nome', 'form_descricao'])
                ->list($form_filters, [
                    'fields' => function ($query) {
                        $query->select(['cam_codigo', 'cam_label', 'cam_tipo', 'cam_opcoes', 'cam_justificativa', 'cafo_sequencial']);
                    }
                ], false);
            //INPUTS
//            $sanitized = [];

            //PESQUISA
            $clients = $this->person_repository->setSelect(['pes_codigo', 'pes_nome'])
                ->setOrder('pes_nome', 'asc')
                ->list([], [
                    'default_address' => function ($query) {
                        $query->select(['pee_codigo', 'pee_logradouro', 'pee_numero', 'pee_bairro', 'pee_cidade',
                            'pee_latitude', 'pee_longitude', 'pee_fk_pes_codigo']);
                    }
                ], false);

            //INPUTS
            $snapshot_filters = ['person' => $person->pes_codigo, 'not_zeroed' => true];

            //PESQUISA
            $inventory = $this->snapshot_inventory_repository
                ->setSelect(['sna_quantidade', 'sna_valor_medio', 'sna_fk_pve_codigo'])
                ->list($snapshot_filters, [
                    'product_variety_pack.measurement_unit' => function ($query) {
                        $query->select(['unm_descricao', 'unm_codigo']);
                    }
                ], false);


            //INPUTS
//            $sanitized = [];

            //PESQUISA
            $branches = $this->branch_repository->setSelect(['fil_codigo', 'fil_descricao'])
                ->setOrder('fil_descricao', 'asc')
                ->list([], [], false);
            //INPUTS
            $product_variety_filters = [
                'only_active' => true,
                'only_service' => true,
            ];

            //PESQUISA
            $available_services = $this->product_variety_pack_repository
                ->list($product_variety_filters, [], false);

            $parameter_filter = [
                'par_codigo' => ['808']
            ];

            //PESQUISA
            $parameters = $this->parameter_repository
                ->setOrder('par_codigo', 'asc')
                ->list($parameter_filter, [], false);

            //PESQUISA
            $address_types = $this->address_type_repository
                ->setSelect(['tpe_codigo as value', 'tpe_descricao as label'])
                ->list([], [], false);

            $inventory_control = $this->inventory_control_scheduling_manager->getScheduling();


                //OUTPUT
            $sync_data = [
                'service_order' => $service_order,
                'forms' => $forms,
                'person' => $person,
                'clients' => $clients,
                'inventory' => $inventory,
                'branches' => $branches,
                'available_services' => $available_services,
                'visitation_reasons' => $visitation_reasons,
                'address_types' => $address_types,
                'parameters' => $parameters,
                'inventory_control' => $inventory_control
            ];

            return new SyncResource($sync_data);
        } catch (\Exception $e) {
            SaveError::generateLog(ErrorLog::MOVEL_GET_SYNC, "", $e->getMessage());
            throw $e;
        }
    }

    public function uploadSync(UploadSyncRequest $request)
    {
        /** @var SyncManager $manager */
        $manager = app()->make(SyncManager::class);
        try {
            $manager->uploadSync($request->validated());
        } catch (\Exception $e) {
            SaveError::generateLog(ErrorLog::MOVEL_SYNC, json_encode($request->all()), $e->getMessage());
            throw $e;
        }
        return response('Ok', 201);
    }
}
