<?php

namespace App\Mobile\Synchronization\Requests;

use App\Domain\Logs\Model\ErrorLog;
use App\Domain\Logs\Service\SaveError;
use Illuminate\Contracts\Validation\Validator;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Validation\ValidationException;

class UploadSyncRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    protected function failedValidation(Validator $validator)
    {
        SaveError::generateLog(ErrorLog::MOVEL_SYNC, json_encode($this->all()), $validator->errors()->toJson());
        parent::failedValidation($validator); // TODO: Change the autogenerated stub
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        return [
            //Validação do cliente
            'clients' => 'nullable|array',
            'clients.*.pes_nome' => 'required|string|max:80',
            'clients.*.pes_identificador' => 'required|string|max:15|identification',
            //Validação do endereço cliente
            'clients.*.address.pee_logradouro' => 'required|string|max:80|nullable',
            'clients.*.address.pee_numero' => 'required|string|max:10|nullable',
            'clients.*.address.pee_bairro' => 'required|string|max:80|nullable',
            'clients.*.address.pee_cep' => 'required|string|max:10|nullable',
            'clients.*.address.pee_pais' => 'required|string|max:30|nullable',
            'clients.*.address.pee_cidade' => 'required|string|max:80|nullable',
            'clients.*.address.pee_uf' => 'required|string|max:2|nullable',
            'clients.*.address.pee_complemento' => 'required|string|max:80|nullable',
            'clients.*.address.pee_latitude' => 'required_with:pee_longitude|numeric|nullable',
            'clients.*.address.pee_longitude' => 'required_with:pee_latitude|numeric|nullable',
            'clients.*.address.pee_fk_tpe_codigo' => 'required|integer|nullable|exists:dynamicdb.cadastro.tipo_endereco,tpe_codigo',
            //Validação dos dados de contato
            "clients.*.contact" => 'required|array',
            'clients.*.contact.peco_contato' => 'required|string|max:80',
            'clients.*.contact.peco_email' => 'required_without:clients.*.contact.peco_telefone|string|max:250|email',
            'clients.*.contact.peco_telefone' => 'required_without:clients.*.contact.peco_email|string|max:15',

            //Validação dos endereços
            'addresses' => 'present|array',
            'addresses.*.pee_codigo' => 'required|integer|exists:dynamicdb.cadastro.pessoa_endereco,pee_codigo',
            'addresses.*.pee_latitude' => 'required|numeric',
            'addresses.*.pee_longitude' => 'required|numeric',

            //Ordens de serviço
            "service_orders" => "present|array",
            "service_orders.*.os_codigo" => "sometimes|integer|exists:dynamicdb.os.os,os_codigo",
            "service_orders.*.os_fk_pes_codigo" => "required_if:service_orders.*.os_concluida,==,true|integer|exists:dynamicdb.cadastro.pessoa,pes_codigo",
            "service_orders.*.os_data_fechamento" => "required_if:service_orders.*.os_concluida,==,true|string|date_format:Y-m-d H:i:s|nullable",
            "service_orders.*.os_fk_fil_codigo" => "required_if:service_orders.*.os_concluida,==,true|integer|nullable|exists:dynamicdb.cadastro.filial,fil_codigo",
            "service_orders.*.os_fk_pee_codigo" => "integer|nullable|exists:dynamicdb.cadastro.pessoa_endereco,pee_codigo",
            "service_orders.*.os_end_cep" => "required_without:service_orders.*.os_fk_pee_codigo|nullable|string|max:10",
            "service_orders.*.os_end_logradouro" => "required_without:service_orders.*.os_fk_pee_codigo|nullable|string|max:80",
            "service_orders.*.os_end_numero" => "required_without:service_orders.*.os_fk_pee_codigo|nullable|string|max:10",
            "service_orders.*.os_end_bairro" => "required_without:service_orders.*.os_fk_pee_codigo|nullable|string|max:80",
            "service_orders.*.os_end_complemento" => "required_without:service_orders.*.os_fk_pee_codigo|nullable|string|max:80",
            "service_orders.*.os_end_cidade" => "required_without:service_orders.*.os_fk_pee_codigo|nullable|string|max:80",
            "service_orders.*.os_end_uf" => "required_without:service_orders.*.os_fk_pee_codigo|nullable|string|max:2",
            //"service_orders.*.os_end_distancia_km" => "required_without:service_orders.*.os_fk_pee_codigo|nullable|numeric|max:1000",
            //Visita
            "service_orders.*.visitation" => "nullable|array",
            "service_orders.*.visitation.vis_observacao" => "required_with:service_orders.*.visitation|string|max:100",
            "service_orders.*.visitation.vis_fk_mov_codigo" => "required_with:service_orders.*.visitation|integer|exists:dynamicdb.movimentacao.motivo_visita,mov_codigo",
            "service_orders.*.visitation.vis_fk_pes_codigo" => "required_with:service_orders.*.visitation|integer|exists:dynamicdb.cadastro.pessoa,pes_codigo",
            "service_orders.*.visitation.vis_data_movel" => "required_with:service_orders.*.visitation|string|date_format:Y-m-d H:i:s",
            //"service_orders.*.visitation.vis_distancia" => "required_if:service_orders.*.os_planejada,false|numeric",
            //Itens da OS
            "service_orders.*.items" => "required_if:service_orders.*.os_concluida,==,true|present|array",
            "service_orders.*.items.*.osps_codigo" => "sometimes|integer|exists:dynamicdb.os.os_produtos_servicos,osps_codigo",
            "service_orders.*.items.*.osps_quantidade" => "required_if:service_orders.*.os_concluida,==,true|numeric",
            "service_orders.*.items.*.osps_servico" => "required_if:service_orders.*.os_concluida,==,true|integer|min:0|max:1",
            "service_orders.*.items.*.osps_fk_pve_codigo" => "required_if:service_orders.*.os_concluida,==,true|integer|exists:dynamicdb.produto.produto_variedade_embalagem,pve_codigo",
            "service_orders.*.items.*.osps_valor_unitario" => "sometimes|numeric",
            //Forms
            "service_orders.*.forms" => "required_if:service_orders.*.os_concluida,==,true|present|array",
            "service_orders.*.forms.*.form_codigo" => "required_if:service_orders.*.os_concluida,==,true|integer|exists:dynamicdb.cadastro.formulario,form_codigo",
            "service_orders.*.forms.*.foa_respostas" => "present|array",
            "service_orders.*.forms.*.foa_respostas.*.cam_codigo" => "required_if:service_orders.*.os_concluida,==,true|integer|exists:dynamicdb.cadastro.campo,cam_codigo",
            "service_orders.*.forms.*.foa_respostas.*.cam_label" => "required_if:service_orders.*.os_concluida,==,true|string",
            "service_orders.*.forms.*.foa_respostas.*.cam_tipo" => "required_if:service_orders.*.os_concluida,==,true|integer|min:0|max:9",
            "service_orders.*.forms.*.foa_respostas.*.foa_resposta" => "required_if:service_orders.*.os_concluida,==,true",
            //PONTO
            "clocks" => "present|array",
            "clocks.*.pnt_data" => "required|string|date_format:Y-m-d H:i:s",
            "clocks.*.pnt_operacao" => "required|integer|min:0|max:1",
        ];
    }
}
